# Python 3.11.1
# 2022-02-01
# Feed sync for UOL Esportes
# Version 0.0.1

## Imports  ##
import json
import requests
import xmltodict
import asyncio
import aiohttp
from xml.dom import minidom
import os
import time
## Functions ##


def getxml():
    url = 'https://pox.globo.com/rss/globorural'
    ua = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7'
    response = requests.get(url, headers={"User-Agent": ua})
    if response.status_code == 200:
        data = xmltodict.parse(response.content)
    return data


def buildXML():
    data = getxml()
    doc = minidom.Document()
    rss = doc.createElement('rss')
    rss.setAttribute('version', '2.0')
    doc.appendChild(rss)
    updtime = doc.createElement('time')
    rss.appendChild(updtime)
    # Get now time #
    now = time.time()
    now = time.ctime(now)
    print(now)
    updtime.appendChild(doc.createTextNode(now))
    channel = doc.createElement('channel')
    rss.appendChild(channel)
    index = 0
    for item in data['rss']['channel']['item']:
        if index > 10:
            break
        # Filter item description if contains blocked words from ../../utils/blocked_words.json #
        # Get blocked words #
        with open('../../utils/blocked_words.json', 'r', encoding='utf-8') as f:
            blocked_words = json.load(f)
            blocked_words = blocked_words['default_words']
        addItem = True
        # Filter description #
        for word in blocked_words:
            if word in item['description']:
                addItem = False
            if word in item['title']:
                addItem = False
        if addItem != False:
            # Title#
            title = doc.createElement('title')
            title.appendChild(doc.createTextNode('Economia'))
            # Description #
            description = doc.createElement('description')
            description.appendChild(doc.createTextNode(item['title']))
            # Path #
            ## Image path = ./images/index.ext ##
            # If media:content exists
            if 'media:content' not in item:
                continue
            else:
                # Get media:content url #
                url = item['media:content']['@url']
                # print(url)
                # Get extension
                ext = url.split('.')[-1]
                # Split ? on ext
                ext = ext.split('?')[0]
                if ext == 'jpg':
                    # Set filename index and ext
                    filename = f'./images/{index}.{ext}'
                    path = doc.createElement('linkfoto')
                    path.appendChild(doc.createTextNode(filename))
                    # Close item #
                    item = doc.createElement('item')
                    item.appendChild(title)
                    item.appendChild(description)
                    item.appendChild(path)
                    channel.appendChild(item)
                    index += 1

    # Save feed.xml UTF8 #
    with open('feed.xml', 'w', encoding='utf-8') as f:
        f.write(doc.toprettyxml(indent='  ', encoding='utf-8').decode('utf-8'))


async def downloadImages():
    start_time = time.time()
    ## Delete all files in folder async ##
    await asyncio.gather(*[asyncio.to_thread(os.remove, f'./images/{file}') for file in os.listdir('./images')])
    buildXML()
    data = getxml()
    index = 0
    async with aiohttp.ClientSession() as session:
        for item in data['rss']['channel']['item']:
            if index > 10:
                break
            # Filter item description if contains blocked words from ../../utils/blocked_words.json #
            # Get blocked words #
            with open('../../utils/blocked_words.json', 'r', encoding='utf-8') as f:
                blocked_words = json.load(f)
                blocked_words = blocked_words['default_words']
            addItem = True
            # Filter description #
            for word in blocked_words:
                if word in item['description']:
                    addItem = False
            if addItem != False:
                # If media:content exists
                if 'media:content' not in item:
                    continue
                else:
                    url = item['media:content']['@url']
                    # Get extension
                    ext = url.split('.')[-1]
                    # Split ? on ext
                    ext = ext.split('?')[0]
                    if ext == 'jpg':
                        # Set filename index and ext
                        filename = f'./images/{index}.{ext}'
                        # Download image
                        async with session.get(url) as response:
                            content = await response.read()
                            with open(filename, 'wb') as f:
                                f.write(content)
                                index += 1
    # Print done with time #
    print("Done! Took {} seconds".format(time.time() - start_time))


if __name__ == '__main__':
    asyncio.run(downloadImages())
